% demo_permutation - demonstrates the differences in "smoothness" between a column stacked     
% version of an image and the result of reordering it using patched-based permutations. 
% We compare the smoothness of the two obtained 1D signals using a moving plot. 
% We also demonstrate how the path through the patches looks like when their reordering  
% is defined via raster scan of the image and patch-based permutation. 
% demo_permutation calculates the patch-based permutations according to Algorithm 1 in:
% I. Ram, M. Elad, and I. Cohen, "Image Processing using Smooth Ordering of its Patches", 
% IEEE Trans. Image Processing, Vol. 22, Issue 7, July 2013, pp. 2764-2774.
% =====================================================================================
% Idan Ram
% Department of Electrical Engineering
% Technion, Haifa 32000 Israel
% idanram@tx.technion.ac.il
%
% November 2013
% =====================================================================================

clc
clear
close all

% rand('seed', 0);
% randn('seed', 0);

% paramters

N = 8; 
B = 0;
eps1 = 10^0.01;
 
% read image and extract patches

origPic = imread( '../images/house.png' );
[ N_m , N_n ] = size( origPic );
N_h_r = floor( N / 2 );
N_h_l = N_h_r;
if mod( N , 2 ) == 0
    N_h_r = N_h_l - 1;
end
bigOrigPic = zeros( N_m + N_h_r + N_h_l );
bigOrigPic( N_h_l + 1 : end - N_h_r , N_h_l + 1 : end - N_h_r ) = origPic;
bigOrigPic( [ 1 : N_h_l , end - N_h_r + 1 : end ] , N_h_l + 1 : end - N_h_r ) = origPic( [ N_h_l:-1:1 , end:-1:end - N_h_r + 1] , : );
bigOrigPic(  : , [ 1 : N_h_l , end - N_h_r + 1 : end ] ) = bigOrigPic( : , [ 2 * N_h_l : -1 : N_h_l + 1 , end - N_h_r : -1 : end - 2 * N_h_r + 1] );
[ N_m_b , N_n_b ] = size( bigOrigPic );
FMat = im2col( bigOrigPic , [ N , N ] , 'sliding' );

% calculate patch-based permutations

X_lLength = size( FMat , 2 );
randStartIx = round( rand * X_lLength ); 
ixPic = reshape( 1 : N_m * N_n , [ N_m , N_n ] );
tempIx = ixPic(:);
[ cur_rows , cur_cols ] = ind2sub( [ N_m , N_n ] , tempIx );
ixMat = [ cur_rows' ; cur_cols' ];
x_restMat = [ FMat ; ones( 1 , X_lLength ) ; -1/2 * sum( FMat.^2 ) ];
x_1Mat = [ FMat ; -1/2 * sum( FMat.^2 ) ; ones( 1 , X_lLength ) ];
randVec = rand( N_m * N_n , 1 );
tic
sortedIx = double( mxCalcPermRand( x_1Mat( : ) , x_restMat( : ) , ixMat( : ) , N^2 + 2 , X_lLength , N_m , randStartIx , B , randVec , eps1 , 1 ) );
toc
% Analysing the results - showing the signal regularity

permOrigPic = origPic( sortedIx ); 
disp('The normalized total variation for the raster-scanned image is:'); 
disp( sum( abs( origPic( 2:end ) - origPic( 1:end - 1 ) ) ) / X_lLength );
disp('The normalized total variation for for the patch-based reordered image is:'); 
disp( sum( abs( permOrigPic( 2:end ) - permOrigPic( 1:end - 1 ) ) ) / X_lLength );

disp('Demonstrating differences in regularity between Raster-Scanned and Patch-Based reordered images'); 

for lag = 0:200:9990
    figure(1); clf;
    plot( 1+lag:1000+lag , origPic( 1+lag:1000+lag ) );
    hold on;
    plot( 1+lag:1000+lag , permOrigPic( 1+lag:1000+lag ) , 'r' );
    axis([lag lag+1000 0 255 ]);
    set(gca,'FontSize',14,'FontName','cabrili');  
    xlabel('coordinate'); 
    ylabel('Gray-Value');
    legend({'Raster-Scanned','Patch-Reordering'});
    pause(0.1); drawnow;
end

% Showing similarity between patches 

disp('Visual demonstration of the path through the patches obtained via Raster-Scan and Patch-Based permutations'); 

lag = 0;
for k = 5501 : 6000
    subplot( 1 , 2 , 1 )
    imshow( reshape( FMat( : , lag + k ) , N , N ) ,[ 0 , 255 ] )
    set(gca,'FontSize',16,'FontName','cabrili');
    title('Raster-Scan')
    subplot( 1 , 2 , 2 )
    imshow( reshape( FMat( : , sortedIx( lag + k ) ) , N , N ) ,[ 0 , 255 ] )
    set(gca,'FontSize',16,'FontName','cabrili');
    title('Patch-Reordering')
    drawnow;
end





